@@ -7,18 +7,20 @@
 import copy
 import math
 import warnings
-from collections import defaultdict
-from typing import DefaultDict, Dict, List, Optional, Tuple
+from collections import defaultdict, deque
+from dataclasses import dataclass, field
+from enum import Enum
+from typing import Any, Callable, DefaultDict, Deque, Dict, List, Optional, Tuple, Union
 
 import torch
 from torch.nn.parameter import UninitializedParameter
+from torch.utils._pytree import PyTree, tree_flatten
+from torch.utils.hooks import RemovableHandle
+from torcheval.tools.flops import flop_mapping, FlopTensorDispatchMode
 
-from torcheval.tools.flops import (
-    flop_mapping,
-    FlopTensor,
-    instrument_module,
-    start_counting,
-)
+from torchtnt.utils import Timer
+from torchtnt.utils.version import is_torch_version_geq_1_13
+from typing_extensions import Literal
 
 _ATTRIB_TO_COL_HEADER = {
     "module_name": "Name",
@@ -26,9 +28,12 @@ _ATTRIB_TO_COL_HEADER = {
     "num_parameters": "# Parameters",
     "num_trainable_parameters": "# Trainable Parameters",
     "size_bytes": "Size (bytes)",
-    "has_uninitialized_param": "Contains Uninitialized Parameter?",
+    "has_uninitialized_param": "Contains Uninitialized Parameters?",
     "flops_forward": "Forward FLOPs",
     "flops_backward": "Backward FLOPs",
+    "in_size": "In size",
+    "out_size": "Out size",
+    "forward_elapsed_time_ms": "Forward Elapsed Times (ms)",
 }  # Attribute: column header (in table)
 _ATTRIBS: List[str] = list(_ATTRIB_TO_COL_HEADER.keys())
 _FLOP_ATTRIBS: List[str] = ["flops_forward", "flops_backward"]
@@ -37,18 +42,39 @@ _FLOP_ATTRIBS: List[str] = ["flops_forward", "flops_backward"]
 _PARAMETER_NUM_UNITS = [" ", "K", "M", "B", "T"]
 _PARAMETER_FLOPS_UNITS = [" ", "k", "M", "G", "T", "P", "E", "Z", "Y"]
 
+TUnknown = Literal["?"]
+_UNKNOWN_SIZE: TUnknown = "?"
+
+
+@dataclass
+class _ModuleSummaryData:
+    # mapping to store forward flops
+    flops_forward: Optional[DefaultDict[str, DefaultDict[str, int]]] = None
+    # mapping to store backward flops
+    flops_backward: Optional[DefaultDict[str, DefaultDict[str, int]]] = None
+    # mapping from module name to activation size tuple (in_size, out_size)
+    activation_sizes: Dict[
+        str, Tuple[Union[TUnknown, List[int]], Union[TUnknown, List[int]]]
+    ] = field(default_factory=dict)
+    # mapping from module name to elapsed time in ms
+    forward_elapsed_times_ms: Dict[str, float] = field(default_factory=dict)
+
 
 class ModuleSummary:
     """
     Summary of module and its submodules. It collects the following information:
+
     - Name
     - Type
     - Number of parameters
     - Number of trainable parameters
     - Estimated size in bytes
-    - Contains uninitialized parameter
+    - Contains uninitialized parameters
     - FLOPs for forward (-1 meaning not calculated)
     - FLOPs for backward (-1 meaning not calculated)
+    - Input shape ("?" meaning not calculated)
+    - Output shape ("?" meaning not calculated)
+    - Forward elapsed time in ms ("?" meaning not calculated)
     """
 
     def __init__(self) -> None:
@@ -63,6 +89,9 @@ class ModuleSummary:
         self._flops_backward: int = -1
         self._flops_forward_detail: Dict[str, int] = {}
         self._flops_backward_detail: Dict[str, int] = {}
+        self._in_size: Union[TUnknown, List[int]] = _UNKNOWN_SIZE
+        self._out_size: Union[TUnknown, List[int]] = _UNKNOWN_SIZE
+        self._forward_time_elapsed_ms: Union[TUnknown, float] = _UNKNOWN_SIZE
 
     @property
     def submodule_summaries(self) -> Dict[str, "ModuleSummary"]:
@@ -125,6 +154,21 @@ class ModuleSummary:
             )
         return self._flops_backward
 
+    @property
+    def in_size(self) -> Union[TUnknown, List[int]]:
+        """Returns the input size of the module"""
+        return self._in_size
+
+    @property
+    def out_size(self) -> Union[TUnknown, List[int]]:
+        """Returns the output size of the module"""
+        return self._out_size
+
+    @property
+    def forward_elapsed_time_ms(self) -> Union[TUnknown, float]:
+        """Returns the forward time of the module in ms."""
+        return self._forward_time_elapsed_ms
+
     @property
     def size_bytes(self) -> int:
         """Returns the total estimated size in bytes of a module."""
@@ -153,39 +197,86 @@ def _clean_flops(flop: DefaultDict[str, DefaultDict[str, int]], N: int) -> None:
             sub_flop[opr] = sub_flop[opr] // N
 
 
-def _get_module_flops(
-    module: torch.nn.Module, module_input: torch.Tensor
-) -> Tuple[
-    DefaultDict[str, DefaultDict[str, int]], DefaultDict[str, DefaultDict[str, int]]
-]:
-    # TODO: here we assume module_input should be a single tensor and module's output should be a single tensor.
-    #   Need to provide more flexible implementation that support other type of input/output.
-    all_hooks: List[torch.utils.hooks.RemovableHandle] = []
-    instrument_module(module, all_hooks, "")
-    module_input = FlopTensor(module_input)
-    N = len(module_input)
+def _get_module_flops_and_activation_sizes(
+    module: torch.nn.Module,
+    # pyre-ignore
+    module_args: Optional[Tuple[Any, ...]] = None,
+    module_kwargs: Optional[Dict[str, Any]] = None,
+) -> _ModuleSummaryData:
+    # a mapping from module name to activation size tuple (in_size, out_size)
+    activation_sizes: Dict[
+        str, Tuple[Union[TUnknown, List[int]], Union[TUnknown, List[int]]]
+    ] = {}
+    # place activation size hooks on all modules + submodules
+    activation_size_handles = _register_hooks(
+        module, [(_activation_size_hook(activation_sizes), _HookType.FORWARD_HOOK)]
+    )
+
+    forward_timer_mapping: Dict[str, Timer] = {}
+    forward_elapsed_times_sec: Dict[str, float] = {}
+    forward_elapsed_time_handles = _register_hooks(
+        module,
+        [
+            (_forward_time_pre_hook(forward_timer_mapping), _HookType.FORWARD_PRE_HOOK),
+            (
+                _forward_time_hook(forward_timer_mapping, forward_elapsed_times_sec),
+                _HookType.FORWARD_HOOK,
+            ),
+        ],
+    )
+
     module.zero_grad()
 
-    # Count for forward flops
-    start_counting()
-    res = module(module_input).mean()
-    flops_forward = copy.deepcopy(FlopTensor.flop_counts)
-    # Count for backward flops
-    start_counting()
-    res.backward()
-    flops_backward = copy.deepcopy(FlopTensor.flop_counts)
+    module_args = module_args or ()
+    module_kwargs = module_kwargs or {}
+    flops_forward = None
+    flops_backward = None
+    if not is_torch_version_geq_1_13():
+        warnings.warn(
+            "Please install PyTorch 1.13 or higher to compute FLOPs: https://pytorch.org/get-started/locally/"
+        )
+        module(*module_args, **module_kwargs)
+        # detach activation size hook handles
+        for hook_handle in activation_size_handles:
+            hook_handle.remove()
+    else:
+        with FlopTensorDispatchMode(module) as ftdm:
+            # Count for forward flops (+ compute activation sizes)
+            res = module(*module_args, **module_kwargs)
 
-    # Norm FLOPs if N>1
-    if N > 1:
-        _clean_flops(flops_forward, N)
-        _clean_flops(flops_backward, N)
+            # detach activation size hook handles
+            for hook_handle in activation_size_handles:
+                hook_handle.remove()
+
+            flops_forward = copy.deepcopy(ftdm.flop_counts)
+            if isinstance(res, torch.Tensor):
+                # Count for backward flops
+                ftdm.reset()
+                res.mean().backward()
+                flops_backward = copy.deepcopy(ftdm.flop_counts)
+            else:
+                warnings.warn(
+                    "Backward FLOPs are only computed if module foward returns a tensor."
+                )
+
+    # remove forward time elapsed handles
+    for hook_handle in forward_elapsed_time_handles:
+        hook_handle.remove()
+
+    # convert module timings to ms
+    forward_time_elapsed_ms = {}
+    for module_name in forward_elapsed_times_sec:
+        forward_time_elapsed_ms[module_name] = (
+            forward_elapsed_times_sec[module_name] / 1000
+        )
 
     # Reverting all the changes:
     module.zero_grad()
+    module_summary_data = _ModuleSummaryData(
+        flops_forward, flops_backward, activation_sizes, forward_time_elapsed_ms
+    )
     # TODO: Reverting BN: We also need to save status of BN running mean/var before running and revert those.
-    for hood_handle in all_hooks:
-        hood_handle.remove()
-    return flops_forward, flops_backward
+    return module_summary_data
 
 
 def _has_uninitialized_param(module: torch.nn.Module) -> bool:
@@ -195,45 +286,60 @@ def _has_uninitialized_param(module: torch.nn.Module) -> bool:
     return False
 
 
+def _has_tensor(item: Optional[PyTree]) -> bool:
+    flattened_list, _ = tree_flatten(item)
+    for ele in flattened_list:
+        if isinstance(ele, torch.Tensor):
+            return True
+    return False
+
+
 def get_module_summary(
     module: torch.nn.Module,
-    module_input: Optional[torch.Tensor] = None,
+    # pyre-ignore
+    module_args: Optional[Tuple[Any, ...]] = None,
+    module_kwargs: Optional[Dict[str, Any]] = None,
 ) -> ModuleSummary:
     """
-    Generate a ModuleSummary object, then assign its values and generate submodule tree.
+    Generate a :class:`~ModuleSummary` object, then assign its values and generate submodule tree.
 
     Args:
         module: The module to be summarized.
-        Value must be greater than 0. Set to `None` to include
-        all submodules.
-        module_name: The name of the current module.
-        module_input: An input for the module to run and calculate FLOPs.
-        Note: if module contains any lazy submodule, we will NOT calculate FLOPs.
-        It should only contain 1 sample (i.e. batch_size=1), otherwise, we will return FLOPs = total FLOPs / len(module_input).
-        Note: currently we only support module that takes a single tensor (i.e. module_input should be a torch.Tensor)
-            and outputs a single tensor. TODO: to support more flexible input and output for module.
+        module_args: A tuple of arguments for the module to run and calculate FLOPs and activation sizes.
+        module_kwargs: Any kwarg arguments to be passed into the module's forward function.
 
+            Note:
+              To calculate FLOPs, you must use PyTorch 1.13 or greater.
+
+            Note:
+              If module contains any lazy submodule, we will NOT calculate FLOPs.
+
+            Note:
+              Currently only modules that output a single tensor are supported.
+              TODO: to support more flexible output for module.
 
     """
 
-    flops_forward = None
-    flops_backward = None
+    module_summary_data = _ModuleSummaryData()
     has_uninitialized_param = _has_uninitialized_param(module)
-    if module_input is not None and not has_uninitialized_param:
-        flops_forward, flops_backward = _get_module_flops(module, module_input)
-    return _generate_module_summary(
-        module,
-        "",
-        flops_forward=flops_forward,
-        flops_backward=flops_backward,
-    )
+    if not has_uninitialized_param:
+        has_tensor_in_args = _has_tensor(module_args)
+        has_tensor_in_kwargs = _has_tensor(module_kwargs)
+        if has_tensor_in_kwargs:
+            warnings.warn(
+                "A tensor in module_kwargs was found. This may lead to an inaccurately computed activation size, as keyword arguments are not passed into forward hooks for modules. "
+                "For best results, please input tensors though module_args."
+            )
+        if has_tensor_in_args or has_tensor_in_kwargs:
+            module_summary_data = _get_module_flops_and_activation_sizes(
+                module, module_args, module_kwargs
+            )
+
+    return _generate_module_summary(module, "", module_summary_data)
 
 
 def _generate_module_summary(
-    module: torch.nn.Module,
-    module_name: str,
-    flops_forward: Optional[DefaultDict[str, DefaultDict[str, int]]] = None,
-    flops_backward: Optional[DefaultDict[str, DefaultDict[str, int]]] = None,
+    module: torch.nn.Module, module_name: str, module_summary_data: _ModuleSummaryData
 ) -> ModuleSummary:
     """
     Recursively generate and populate metrics for ModelSummary.
@@ -247,10 +353,7 @@ def _generate_module_summary(
         formatted_name = f"{module_name}.{name}" if module_name != "" else name
 
         submodule_summary = _generate_module_summary(
-            submodule,
-            formatted_name,
-            flops_forward=flops_forward,
-            flops_backward=flops_backward,
+            submodule, formatted_name, module_summary_data
         )
 
         # Add results from submodule summary
@@ -278,6 +381,11 @@ def _generate_module_summary(
     for buf in module.buffers(recurse=False):
         module_summary._size_bytes += buf.numel() * buf.element_size()
 
+    flops_forward = module_summary_data.flops_forward
+    flops_backward = module_summary_data.flops_backward
+    activation_sizes = module_summary_data.activation_sizes
+    forward_elapsed_times_ms = module_summary_data.forward_elapsed_times_ms
+
     # Calculate flops forward/backward.
     if flops_forward is not None:
         module_summary._flops_forward_detail = dict(flops_forward[module_name])
@@ -290,6 +398,16 @@ def _generate_module_summary(
             [v for k, v in flops_backward[module_name].items()]
         )
 
+    # set activation sizes
+    if module_name in activation_sizes:
+        in_size, out_size = activation_sizes[module_name]
+        module_summary._in_size = in_size
+        module_summary._out_size = out_size
+
+    # set forward elasped times
+    if module_name in forward_elapsed_times_ms:
+        module_summary._forward_time_elapsed_ms = forward_elapsed_times_ms[module_name]
+
     return module_summary
 
 
@@ -309,6 +427,12 @@ def get_summary_table(
         stop_attr.append("flops_forward")
     if module_summary.flops_backward == -1:
         stop_attr.append("flops_backward")
+    if module_summary.in_size == _UNKNOWN_SIZE:
+        stop_attr.append("in_size")
+    if module_summary.out_size == _UNKNOWN_SIZE:
+        stop_attr.append("out_size")
+    if module_summary.forward_elapsed_time_ms == _UNKNOWN_SIZE:
+        stop_attr.append("forward_elapsed_time_ms")
     unpacked_attribs, col_widths = defaultdict(list), defaultdict(int)
     _unpack_attributes(
         {"root": module_summary},
@@ -327,7 +451,7 @@ def get_summary_table(
     header = [
         s.format(col_header, col_width)
         for col_header, col_width in zip(
-            _ATTRIB_TO_COL_HEADER.values(), col_widths.values()
+            [_ATTRIB_TO_COL_HEADER[attr] for attr in use_attribs], col_widths.values()
         )
     ]
     summary_table = " | ".join(header) + "\n" + "-" * total_width + "\n"
@@ -432,6 +556,10 @@ def _unpack_attributes(
                     if human_readable_nums
                     else str(attrib_val)
                 )
+            elif isinstance(attrib_val, float):
+                attrib_val = f"{attrib_val:.10f}"
+            elif isinstance(attrib_val, list):
+                attrib_val = str(attrib_val)
             elif attrib_val is None:
                 attrib_val = ""
 
@@ -501,3 +629,115 @@ def _get_human_readable_count(number: int, labels: Optional[List[str]] = None) -
         return f"{int(number):,d} {labels[index]}"
 
     return f"{number:,.1f} {labels[index]}"
+
+
+def _parse_batch_shape(batch: torch.Tensor) -> Union[TUnknown, List[int]]:
+    if hasattr(batch, "shape"):
+        return list(batch.shape)
+
+    if isinstance(batch, (list, tuple)):
+        shape = [_parse_batch_shape(el) for el in batch]
+        return shape
+
+    return _UNKNOWN_SIZE
+
+
+class _HookType(Enum):
+    FORWARD_PRE_HOOK = 1
+    FORWARD_HOOK = 2
+    BACKWARD_PRE_HOOK = 3
+    BACKWARD_HOOK = 4
+
+
+def _activation_size_hook(
+    activation_sizes: Dict[
+        str, Tuple[Union[TUnknown, List[int]], Union[TUnknown, List[int]]]
+    ],
+    # pyre-ignore: Invalid type parameters [24]
+) -> Callable[[str], Callable]:
+    # pyre-ignore: Missing parameter annotation [2]
+    def intermediate_hook(
+        module_name: str,
+    ) -> Callable[[torch.nn.Module, Any, Any], None]:
+        # pyre-ignore
+        def hook(_: torch.nn.Module, inp: Any, out: Any) -> None:
+            if len(inp) == 1:
+                inp = inp[0]
+            in_size = _parse_batch_shape(inp)
+            out_size = _parse_batch_shape(out)
+            activation_sizes[module_name] = (in_size, out_size)
+
+        return hook
+
+    return intermediate_hook
+
+
+def _forward_time_pre_hook(
+    timer_mapping: Dict[str, Timer]
+    # pyre-ignore: Invalid type parameters [24]
+) -> Callable[[str], Callable]:
+    # pyre-ignore: Missing parameter annotation [2]
+    def intermediate_hook(
+        module_name: str,
+    ) -> Callable[[torch.nn.Module, Any], None]:
+        def hook(_module: torch.nn.Module, _inp: Any) -> None:
+            timer_mapping[module_name] = Timer()
+            timer_mapping[module_name].start()
+
+        return hook
+
+    return intermediate_hook
+
+
+def _forward_time_hook(
+    timer_mapping: Dict[str, Timer],
+    elapsed_times: Dict[str, float],
+    # pyre-ignore: Invalid type parameters [24]
+) -> Callable[[str], Callable]:
+    # pyre-ignore: Missing parameter annotation [2]
+    def intermediate_hook(
+        module_name: str,
+    ) -> Callable[[torch.nn.Module, Any, Any], None]:
+        def hook(_module: torch.nn.Module, _inp: Any, _out: Any) -> None:
+            timer_mapping[module_name].stop()
+            elapsed_times[module_name] = timer_mapping[
+                module_name
+            ].interval_time_seconds
+
+        return hook
+
+    return intermediate_hook
+
+
+def _register_hooks(
+    module: torch.nn.Module,
+    # pyre-ignore: Invalid type parameters [24]
+    hooks: List[Tuple[Callable, _HookType]],
+) -> List[RemovableHandle]:
+    """
+    Handles recursive hook attachment to module and its submodules, and passes the module name to each hook function.
+    """
+    removable_handle_list: List[RemovableHandle] = []
+    if not isinstance(module, torch.jit.ScriptModule):
+        queue: Deque[Tuple[str, torch.nn.Module]] = deque([("", module)])
+        while len(queue) > 0:
+            module_name, mod = queue.pop()
+
+            # register hook
+            for hook, hook_type in hooks:
+                if hook_type == _HookType.FORWARD_PRE_HOOK:
+                    handle = mod.register_forward_pre_hook(hook(module_name))
+                elif hook_type == _HookType.FORWARD_HOOK:
+                    handle = mod.register_forward_hook(hook(module_name))
+                elif hook_type == _HookType.BACKWARD_PRE_HOOK:
+                    handle = mod.register_full_backward_pre_hook(hook(module_name))
+                else:
+                    handle = mod.register_full_backward_hook(hook(module_name))
+                removable_handle_list.append(handle)
+
+            for name, submodule in mod.named_children():
+                formatted_name = f"{module_name}.{name}" if module_name != "" else name
+                queue.append((formatted_name, submodule))
+    else:
+        warnings.warn("Registering hooks on torch.jit.ScriptModule is not supported.")
+    return removable_handle_list
